from django import template

from ..models import Property, Value

SI_PREFIXES = {
    -12: "p",
    -9: "n",
    -6: "µ",
    -3: "m",
    3: "k",
    6: "M",
    9: "G",
}

register = template.Library()


@register.filter
def format_value(value):
    if not isinstance(value, Value):
        return value

    property = value.property
    match property.type:
        case Property.Type.QUANTITY:
            s = str(value.integer)

            scale = property.scale
            if scale > 0:
                s += "0" * scale
            elif scale < 0:
                s = str(value.integer).rjust(abs(scale) + 1, "0")
                s = s[:scale] + "." + s[scale:]
                s = s.rstrip("0").removesuffix(".")

            leading_zeroes = len(s.removeprefix("0.")) - len(
                s.removeprefix("0.").lstrip("0")
            )
            if prefix := SI_PREFIXES.get(-leading_zeroes // 3 * 3, ""):
                s = s.removeprefix("0.").lstrip("0")
                s = (s[: -leading_zeroes % 3] or "0") + "." + s[-leading_zeroes % 3 :]

            trailing_zeroes = len(s) - len(s.rstrip("0"))
            if prefix := SI_PREFIXES.get(trailing_zeroes // 3 * 3, ""):
                s = s.rstrip("0") + "0" * (trailing_zeroes % 3)

            return f"{s} {prefix}{value.property.unit}"
        case Property.Type.TEXT:
            return value.text
